% Function calculate the quantile for particular weights
   
function result = quantile_weight(weights, price_scenario_RB, price_scenario_RDN, alpha, measure, varargin)

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Input data %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % weights - vector of weights
    % price_scenario_RB - prediction prices for the complementary market
    % price_scenario_RDN - prediction prices for the day-ahead market
    % alpha - order of risk measure
    % k - number of points in the third window
    % nr_U - number of scenarios
    % measure - measure of risk

    minArgs = 5;
    maxArgs = 6;
    narginchk(minArgs, maxArgs)
    if nargin == 6
        transaction_side = varargin{1};
    end

    nr_U = length(price_scenario_RB);
    scenario_h_t = zeros(length(weights), nr_U);
    quantile_weights = zeros(1,length(weights));
    
    switch measure
        case 'quantile'
        for i = 1:length(weights)
            scenario_h_t(i,:) = scenario(weights, price_scenario_RB, price_scenario_RDN, i);
            quantile_weights(i) = quantile(scenario_h_t(i,:), alpha);
        end
        case 'expectiles'
            for i = 1:length(weights)
                scenario_h_t(i,:) = scenario(weights, price_scenario_RB, price_scenario_RDN, i);
                quantile_weights(i) = expectile(scenario_h_t(i,:),alpha);
            end
        case 'ES' 
            for i = 1:length(weights)
                scenario_h_t(i,:) = scenario(weights, price_scenario_RB, price_scenario_RDN, i);
                VaR = quantile(scenario_h_t(i,:),alpha);
                if alpha == 0.05
                    index = find(scenario_h_t(i,:) < VaR);
                    help_vec = zeros(1,length(index));
                    for z = 1:length(index)
                        help_vec(z) = scenario_h_t(i,index(z));
                    end
                    quantile_weights(i) = mean(help_vec);
                end
                if alpha == 0.95
                    index = find(scenario_h_t(i,:) > VaR);
                    help_vec = zeros(1,length(index));
                    for z = 1:length(index)
                        help_vec(z) = scenario_h_t(i,index(z));
                    end
                    quantile_weights(i) = mean(help_vec);
                end
            end
        case 'sharpe'
            for i = 1:length(weights)
                scenario_h_t(i,:) = scenario(weights, price_scenario_RB, price_scenario_RDN, i);
                if -mean(scenario_h_t(i,:)) > 0 && strcmp(transaction_side, 'buyer')
                    quantile_weights(i) = mean(scenario_h_t(i,:)) * std(scenario_h_t(i,:)); % for buyer we want to minimize both std and mean
                elseif mean(scenario_h_t(i,:)) > 0 && strcmp(transaction_side, 'seller')
                    quantile_weights(i) = mean(scenario_h_t(i,:)) / std(scenario_h_t(i,:));
                elseif -mean(scenario_h_t(i,:)) < 0 && strcmp(transaction_side, 'buyer')
                    quantile_weights(i) = mean(scenario_h_t(i,:)) / std(scenario_h_t(i,:)); % for buyer we want to minimize both std and mean
                elseif mean(scenario_h_t(i,:)) < 0 && strcmp(transaction_side, 'seller')
                    quantile_weights(i) = mean(scenario_h_t(i,:)) * std(scenario_h_t(i,:)); % for buyer we want to minimize both std and mean
                end
            end
        case 'q_50_IQR'
            for i = 1:length(weights)
                scenario_h_t(i,:) = scenario(weights, price_scenario_RB, price_scenario_RDN, i);
                quantile_weights(i) = quantile(scenario_h_t(i,:), 0.5)/iqr(scenario_h_t(i,:));
            end   
        case 'median'
            for i = 1:length(weights)
                scenario_h_t(i,:) = scenario(weights, price_scenario_RB, price_scenario_RDN, i);
                quantile_weights(i) = quantile(scenario_h_t(i,:), 0.5);
            end 
        case {'mean', 'mean_and_std'}
            for i = 1 :length(weights)
                scenario_h_t(i,:) = scenario(weights, price_scenario_RB, price_scenario_RDN, i);
                quantile_weights(i) = mean(scenario_h_t(i,:));
            end 
        case {'std', 'std_and_profit'}
            for i = 1:length(weights)
                scenario_h_t(i, :) = scenario(weights, price_scenario_RB, price_scenario_RDN, i);
                quantile_weights(i) = std(scenario_h_t(i, :));
            end
        case 'semistd_down'
            for i = 1:length(weights)
                scenario_h_t(i, :) = scenario(weights, price_scenario_RB, price_scenario_RDN, i);
                quantile_weights(i) = semistd(scenario_h_t(i, :), -1);
            end
        case 'semistd_up'
            for i = 1:length(weights)
                scenario_h_t(i, :) = scenario(weights, price_scenario_RB, price_scenario_RDN, i);
                quantile_weights(i) = semistd(scenario_h_t(i, :), 1);
            end
    end
    result = quantile_weights;
end
